@@ -3,23 +3,31 @@
 //! It initializes the memory allocation and logging crates,
 //! allowing code to use Rust's data structures and to log errors.
 //!
+//! Logging and allocation are only allowed while boot services are
+//! active. Once runtime services are activated by calling
+//! [`exit_boot_services`], the logger will be disabled and the
+//! allocator will always return null.
+//!
 //! It also stores a global reference to the UEFI system table,
 //! in order to reduce the redundant passing of references to it.
 //!
 //! Library code can simply use global UEFI functions
 //! through the reference provided by `system_table`.
+//!
+//! [`exit_boot_services`]: uefi::table::SystemTable::exit_boot_services
 
 #![no_std]
 #![feature(alloc_error_handler)]
-#![feature(asm)]
 #![feature(lang_items)]
 #![feature(panic_info_message)]
+#![feature(abi_efiapi)]
 
 #[macro_use]
 extern crate log;
 // Core types.
 extern crate uefi;
 
+use core::ffi::c_void;
 use core::ptr::NonNull;
 
 use cfg_if::cfg_if;
@@ -81,6 +89,7 @@ pub fn init(st: &mut SystemTable<Boot>) -> Result {
                 EventType::SIGNAL_EXIT_BOOT_SERVICES,
                 Tpl::NOTIFY,
                 Some(exit_boot_services),
+                None,
             )
             .map_inner(|_| ())
     }
@@ -107,19 +116,19 @@ unsafe fn init_logger(st: &mut SystemTable<Boot>) {
 }
 
 /// Notify the utility library that boot services are not safe to call anymore
-fn exit_boot_services(_e: Event) {
+/// As this is a callback, it must be `extern "efiapi"`.
+unsafe extern "efiapi" fn exit_boot_services(_e: Event, _ctx: Option<NonNull<c_void>>) {
     // DEBUG: The UEFI spec does not guarantee that this printout will work, as
     //        the services used by logging might already have been shut down.
     //        But it works on current OVMF, and can be used as a handy way to
     //        check that the callback does get called.
     //
     // info!("Shutting down the UEFI utility library");
-    unsafe {
-        SYSTEM_TABLE = None;
-        if let Some(ref mut logger) = LOGGER {
-            logger.disable();
-        }
+    SYSTEM_TABLE = None;
+    if let Some(ref mut logger) = LOGGER {
+        logger.disable();
     }
+
     uefi::alloc::exit_boot_services();
 }
 
@@ -177,14 +186,14 @@ fn panic_handler(info: &core::panic::PanicInfo) -> ! {
                     loop {
                         unsafe {
                             // Try to at least keep CPU from running at 100%
-                            asm!("hlt", options(nomem, nostack));
+                            core::arch::asm!("hlt", options(nomem, nostack));
                         }
                     }
                 } else if #[cfg(target_arch = "aarch64")] {
                     loop {
                         unsafe {
                             // Try to at least keep CPU from running at 100%
-                            asm!("hlt 420", options(nomem, nostack));
+                            core::arch::asm!("hlt 420", options(nomem, nostack));
                         }
                     }
                 } else {
